# EKF - Linear System Model

This document describes the use of the Extended Kalman Filter (EKF) in the REXYGEN environment for state estimation of a linear system.

## System Description

The system is described in accordance with the documentation of the [EKF Block in REXYGEN](https://www.rexygen.com/doc/ENGLISH/MANUALS/BRef/EKF.html#x229-22800013), where can be found further details.

### State Model of the System

The system dynamics are defined by the function *f(x(t), u(t))*:  
*f(x(t), u(t)) = [dx1(t)/dt; dx2(t)/dt; dx3(t)/dt; dx4(t)/dt]*, where  
*dx1(t)/dt = -x1(t) + x2(t)*  
*dx2(t)/dt = -x2(t) + x3(t)*  
*dx3(t)/dt = -x3(t) + x4(t)*  
*dx4(t)/dt = -x4(t) + u1(t)*

### Jacobian of the State Function

The Jacobian *f(x(t), u(t))* with respect to the state vector *x* is given as:  
*df(x, u)/dx = [-1 1 0 0; 0 -1 1 0; 0 0 -1 1; 0 0 0 -1]*

### Output Measurements

The first set of output measurements (which, in this example, depend linearly on the state variables) for input **EKF:nz = 1** is defined as:  
*y(t) = h1(x(t), u(t)) = [x1(t); 2 x2(t) + 0.5 x4(t)]*  
with the corresponding Jacobian:  
*dh1(x(t), u(t))/dt = [1 0 0 0; 0 2 0 0.5]*

The second set of output measurements (for input **EKF:nz = 2**) is not used.

---

The system description is programmed into the REXLANG block.

## Timing of the project ##
The algorithm runs each 40 milliseconds (0.04 s). See the EXEC function block,  
tick x ntick0 = 0.02 x 2 = 0.04.

## Running the examples ##
- The **exec.mdl* file is the project main file.
- Open it with *REXYGEN Studio*, compile and download it to the target device.
- Observe the mean state vector with *TRNDV_X*, *VTOR_X* blocks and trace of the of the state estimation covariance matrix with *TRNDV_trP* and *VTOR_trP* blocks.